//**********************************************************************
// This code was created by SoC Solutions and is free for use with     *
// Actel products.  For support e-mail support@socsolutions.com.       *
//**********************************************************************

#include "socDefs.h"
#include "APB_Analog.h"
#include "acm_defines.h"
#include "quads_acm_cfg.h"

#ifdef DEBUG_ANALOG
#include "SocAddrDec.h"
#endif

static uint8 adc_stc;

///////////////////////////////////////////////////////////////////////////////// 
// These functions control and read data from the A/D Converter.
///////////////////////////////////////////////////////////////////////////////// 

// make sure no calibration or conversion in progress
int adc_poll_ready(uint32 ai_base_addr)
{
    int i;
    uint16 status;

    // make sure a read or write operation has completed
    for (i = 1; i <= MAX_ADC_POLLS; i++)
    {
        GET_UINT16(ai_base_addr+ANALOG_ADC_STAT_OFFSET, &status);

        if (!(status & (ADC_STAT_CALIBRATE_MASK | ADC_STAT_SAMPLE_MASK | ADC_STAT_BUSY_MASK)))
            return(0);
        else if (i >= MAX_ADC_POLLS)
        {
#ifdef DEBUG_ANALOG
            uart_puts(UARTA_BASE, "\nToo many ADC polls!\n");
#endif
            return(-1);
        }
    }
    return(0);
}

///////////////////////////////////////////////////////////////////////////////// 
// These functions manipulate the ACM interface to perform reads & writes for the 
// analog quads & RTC on Actel Fusion devices.
///////////////////////////////////////////////////////////////////////////////// 

// make sure a previous read or write has completed
int acm_poll_ready(uint32 ai_base_addr, uint8 busy_mask)
{
    int i;
    uint16 status;

    // make sure a read or write operation has completed
    for (i = 1; i <= MAX_ACM_POLLS; i++)
    {
        GET_UINT16(ai_base_addr+ANALOG_ACM_CTRL_STAT_OFFSET, &status);
        if (!((uint8) status & busy_mask))
            return(0);
        else if (i >= MAX_ACM_POLLS)
            return(-1);
    }
    return(0);
}

// Read a byte from an ACM register.
int acm_get_uint8(uint32 ai_base_addr, uint8 acm_offset, uint8 *pData)
{
    uint16 data16;

    // make sure a previous read or write operation has completed
    if (acm_poll_ready(ai_base_addr, ACMWRBUSY_MASK | ACMRDBUSY_MASK))
        return(-1);

    // write the ACM address
    PUT_UINT16(ai_base_addr+ANALOG_ACM_ADDR_OFFSET, (uint16) acm_offset);

    // start the read operation
    PUT_UINT16(ai_base_addr+ANALOG_ACM_CTRL_STAT_OFFSET, ACMRDSTART_MASK);

    // wait for read operation to complete
    if (acm_poll_ready(ai_base_addr, ACMRDBUSY_MASK))
        return(-1);

    // read the data
    GET_UINT16(ai_base_addr+ANALOG_ACM_DATA_OFFSET, &data16);
    *pData = (uint8) data16;

    return(0);
}

// Read a 16-bit word from an ACM register.
int acm_get_uint16(uint32 ai_base_addr, uint8 acm_offset, uint16 *pData)
{
    int i;
    uint8 *pData8;
    uint16 data16;

    pData8 = (uint8 *) pData;

    for (i = 0; i < 2; i++)
    {
        // make sure a previous read or write operation has completed
        if (acm_poll_ready(ai_base_addr, ACMWRBUSY_MASK | ACMRDBUSY_MASK))
            return(-1);

        // write the ACM address
        PUT_UINT16(ai_base_addr+ANALOG_ACM_ADDR_OFFSET, (uint16) (acm_offset+i));

        // start the read operation
        PUT_UINT16(ai_base_addr+ANALOG_ACM_CTRL_STAT_OFFSET, ACMRDSTART_MASK);

        // wait for read operation to complete
        if (acm_poll_ready(ai_base_addr, ACMRDBUSY_MASK))
            return(-1);

        // read the data
        GET_UINT16(ai_base_addr+ANALOG_ACM_DATA_OFFSET, &data16);
        *(pData8+i) = (uint8) data16;
    }
    return(0);
}

// Read a 32-bit word from an ACM register.
int acm_get_uint32(uint32 ai_base_addr, uint8 acm_offset, uint32 *pData)
{
    int i;
    uint8 *pData8;
    uint16 data16;

    pData8 = (uint8 *) pData;

    for (i = 0; i < 4; i++)
    {
        // make sure a previous read or write operation has completed
        if (acm_poll_ready(ai_base_addr, ACMWRBUSY_MASK | ACMRDBUSY_MASK))
            return(-1);

        // write the ACM address
        PUT_UINT16(ai_base_addr+ANALOG_ACM_ADDR_OFFSET, (uint16) (acm_offset+i));

        // start the read operation
        PUT_UINT16(ai_base_addr+ANALOG_ACM_CTRL_STAT_OFFSET, ACMRDSTART_MASK);

        // wait for read operation to complete
        if (acm_poll_ready(ai_base_addr, ACMRDBUSY_MASK))
            return(-1);

        // read the data
        GET_UINT16(ai_base_addr+ANALOG_ACM_DATA_OFFSET, &data16);
        *(pData8+i) = (uint8) data16;
    }
    return(0);
}

// Write a byte to an ACM register.
int acm_put_uint8(uint32 ai_base_addr, uint8 acm_offset, uint8 data)
{
    // make sure a previous read or write operation has completed
    if (acm_poll_ready(ai_base_addr, ACMWRBUSY_MASK | ACMRDBUSY_MASK))
        return(-1);

    // write the ACM address
    PUT_UINT16(ai_base_addr+ANALOG_ACM_ADDR_OFFSET, (uint16) acm_offset);

    // write the data
    PUT_UINT16(ai_base_addr+ANALOG_ACM_DATA_OFFSET, (uint16) data);

    return(0);
}

// Write a 16-bit word to an ACM register.
int acm_put_uint16(uint32 ai_base_addr, uint8 acm_offset, uint16 data)
{
    int i;
    uint16 data16;
    uint8 *pData8;

    data16 = data;
    pData8 = (uint8 *) &data16;

    for (i = 0; i < 2; i++)
    {
        // make sure a previous read or write operation has completed
        if (acm_poll_ready(ai_base_addr, ACMWRBUSY_MASK | ACMRDBUSY_MASK))
            return(-1);

        // write the ACM address
        PUT_UINT16(ai_base_addr+ANALOG_ACM_ADDR_OFFSET, (uint16) (acm_offset+i));

        // write the data
        PUT_UINT16(ai_base_addr+ANALOG_ACM_DATA_OFFSET, (uint16) (*pData8++));
    }
    return(0);
}

// Write a 32-bit word to an ACM register.
int acm_put_uint32(uint32 ai_base_addr, uint8 acm_offset, uint32 data)
{
    int i;
    uint32 data32;
    uint8 *pData8;

    data32 = data;
    pData8 = (uint8 *) &data32;

    for (i = 0; i < 4; i++)
    {
        // make sure a previous read or write operation has completed
        if (acm_poll_ready(ai_base_addr, ACMWRBUSY_MASK | ACMRDBUSY_MASK))
            return(-1);

        // write the ACM address
        PUT_UINT16(ai_base_addr+ANALOG_ACM_ADDR_OFFSET, (uint16) (acm_offset+i));

        // write the data
        PUT_UINT16(ai_base_addr+ANALOG_ACM_DATA_OFFSET, (uint16) (*pData8++));
    }
    return(0);
}

int adc_init(uint32 ai_base_addr)
{
#ifdef DEBUG_ANALOG
    uint32 acm_data;
#endif
    adc_stc = 64;   // should be at least 53 to give 10 uS at 22 MHz / 4 ADC clock

    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL1_OFFSET, ADC_PWRDN_MASK);
    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL1_OFFSET, 0);
    if (adc_poll_ready(ai_base_addr))   // make sure ADC is idle
        return(-1);
    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL1_OFFSET, ADC_RESET_MASK);
    if (adc_poll_ready(ai_base_addr))   // make sure ADC is idle
        return(-1);

    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL1_OFFSET, ADC_MODE);     // TVC = 0
    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL2_OFFSET, (uint16) adc_stc);

    // init ACM registers
    acm_put_uint32(ai_base_addr, ACM_AQ0_CTRL, CFG_QUAD0);
    acm_put_uint32(ai_base_addr, ACM_AQ1_CTRL, CFG_QUAD1);
    acm_put_uint32(ai_base_addr, ACM_AQ2_CTRL, CFG_QUAD2);
    acm_put_uint32(ai_base_addr, ACM_AQ3_CTRL, CFG_QUAD3);
    acm_put_uint32(ai_base_addr, ACM_AQ4_CTRL, CFG_QUAD4);
    acm_put_uint32(ai_base_addr, ACM_AQ5_CTRL, CFG_QUAD5);
    acm_put_uint32(ai_base_addr, ACM_AQ6_CTRL, CFG_QUAD6);
    acm_put_uint32(ai_base_addr, ACM_AQ7_CTRL, CFG_QUAD7);
    acm_put_uint32(ai_base_addr, ACM_AQ8_CTRL, CFG_QUAD8);
    acm_put_uint32(ai_base_addr, ACM_AQ9_CTRL, CFG_QUAD9);

#ifdef DEBUG_ANALOG
    // verify register contents (debug only)
    acm_get_uint32(ai_base_addr, ACM_AQ0_CTRL, &acm_data);
    if (acm_data != CFG_QUAD0)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ1_CTRL, &acm_data);
    if (acm_data != CFG_QUAD1)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ2_CTRL, &acm_data);
    if (acm_data != CFG_QUAD2)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ3_CTRL, &acm_data);
    if (acm_data != CFG_QUAD3)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ4_CTRL, &acm_data);
    if (acm_data != CFG_QUAD4)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ5_CTRL, &acm_data);
    if (acm_data != CFG_QUAD5)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ6_CTRL, &acm_data);
    if (acm_data != CFG_QUAD6)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ7_CTRL, &acm_data);
    if (acm_data != CFG_QUAD7)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ8_CTRL, &acm_data);
    if (acm_data != CFG_QUAD8)
        return(-1);
    acm_get_uint32(ai_base_addr, ACM_AQ9_CTRL, &acm_data);
    if (acm_data != CFG_QUAD9)
        return(-1);
#endif

    // init RTC reg's here
    return(0);
}

/*********************************************************
The steps for performing an ADC conversion are as follows:
1. Read the ADC Status register to ensure that the ADC
is not busy calibrating or performing a conversion.
2. Write the desired ADC settings to ADC Control
register 1, including the voltage reference selection
(internal or external), mode selection (ADC
resolution), and clock divider settings.
3. Write the desired ADC settings to ADC Control
register 2, including the sample time control and
channel number to sample (1 of 32).
4. Set the Start Conversion bit to begin the
conversion process.
5. If a current or temperature monitor operation is
used on a corresponding AC or AT input pin, a write
must be done to set the corresponding current or
temperature monitor strobe high in ADC Control
register 3 or ADC Control register 4, respectively.
Refer to the Fusion datasheet for details on
external component connection requirements for
current and temperature monitoring.
6. Keep reading the high-order ADC Status register
until the ADC is not busy performing a conversion.
7. If a current or temperature monitor operation has
completed using a corresponding AC or AT input
pin, a write must be done to set the corresponding
current or temperature monitor strobe low in ADC
Control register 3 or ADC Control register 4,
respectively.
8. Read the ADC Status register to obtain
the resulting ADC conversion data.
*********************************************************/

uint16 get_current_ch_mask(uint8 amux_ch)
{
    uint16 mask;
    switch (amux_ch)
    {
        case AMUX_CH_AC0: mask = 0x0001; break;
        case AMUX_CH_AC1: mask = 0x0002; break;
        case AMUX_CH_AC2: mask = 0x0004; break;
        case AMUX_CH_AC3: mask = 0x0008; break;
        case AMUX_CH_AC4: mask = 0x0010; break;
        case AMUX_CH_AC5: mask = 0x0020; break;
        case AMUX_CH_AC6: mask = 0x0040; break;
        case AMUX_CH_AC7: mask = 0x0080; break;
        case AMUX_CH_AC8: mask = 0x0100; break;
        case AMUX_CH_AC9: mask = 0x0200; break;
        default:          mask = 0x0000; break;
    }
    return(mask);
}

uint16 get_temp_ch_mask(uint8 amux_ch)
{
    uint16 mask;
    switch (amux_ch)
    {
        case AMUX_CH_AT0:        mask = 0x0001; break;
        case AMUX_CH_AT1:        mask = 0x0002; break;
        case AMUX_CH_AT2:        mask = 0x0004; break;
        case AMUX_CH_AT3:        mask = 0x0008; break;
        case AMUX_CH_AT4:        mask = 0x0010; break;
        case AMUX_CH_AT5:        mask = 0x0020; break;
        case AMUX_CH_AT6:        mask = 0x0040; break;
        case AMUX_CH_AT7:        mask = 0x0080; break;
        case AMUX_CH_AT8:        mask = 0x0100; break;
        case AMUX_CH_AT9:        mask = 0x0200; break;
        case AMUX_CH_INT_TEMP:   mask = 0x0400; break;
        default:                 mask = 0x0000; break;
    }
    return(mask);
}

// perform an A/D measurement on the specified channel
int adc_meas(uint32 ai_base_addr, uint8 amux_ch, uint16 ch_type, uint16 *result)
{
    uint16 status;

    if (adc_poll_ready(ai_base_addr))   // make sure ADC is idle
        return(-1);

    // set a strobe if necessary
    if (ch_type == ANALOG_CH_TYPE_CURRENT)
        PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL3_OFFSET, get_current_ch_mask(amux_ch));
    else if (ch_type == ANALOG_CH_TYPE_TEMP)
        PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL4_OFFSET, get_temp_ch_mask(amux_ch));

    // start the conversion on the selected channel
    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL2_OFFSET, (uint16) (ADC_START_MASK | (amux_ch << 8) | adc_stc));

    if (adc_poll_ready(ai_base_addr))   // make sure ADC is idle
        return(-1);

    GET_UINT16(ai_base_addr+ANALOG_ADC_STAT_OFFSET, &status);   // read valid status & data

    // clear a strobe if necessary
    if (ch_type == ANALOG_CH_TYPE_CURRENT)
        PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL3_OFFSET, 0);
    else if (ch_type == ANALOG_CH_TYPE_TEMP)
        PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL4_OFFSET, 0);

    // if data is valid
    if (status & ADC_STAT_DATAVALID_MASK)    
    {   
    	*result = status & ADC_STAT_RESULT_MASK;
		
        // adjust data for significant bits if necessary
        if (ADC_MODE == ADC_MODE_10BIT)
            *result = *result >> 2;                       
        else if (ADC_MODE == ADC_MODE_8BIT)
            *result = *result >> 4;            
        return(0);
    }
    else
        return(-1);
}

void set_gate_driver(uint32 ai_base_addr, uint16 gate_drv_mask)
{
    // write the mask to enable/disable the selected channels
    // Note that 1 turns driver on, sinking current, driving output low
    // so bits are inverted before writing to register.
    PUT_UINT16(ai_base_addr+ANALOG_ADC_CTRL5_OFFSET, ~gate_drv_mask);
}

