//**********************************************************************
// This code was created by SoC Solutions and is free for use with     *
// Actel products.  For support e-mail support@socsolutions.com.       *
//**********************************************************************

#include "socDefs.h"
#include "socTb.h"
#include "SocAddrDec.h"
#include "APB_UART.h"
#include "APB_Analog.h"
#include "externals.h"

//#include <string.h>
//#include <stdlib.h>
//#include <stdio.h>

char * unsignedToAscii (unsigned val, unsigned width)
{
    static char rtnVal[9];
    int i;

    for(i=0; i<9; i++) rtnVal[i] = 0;
    for(i=0; i<8; i++)
    {
        if (((val>>28) & 0xf) < 10) rtnVal[i] = (((val>>28) & 0xf) + 48);
        else                        rtnVal[i] = (((val>>28) & 0xf) + 55);

        val = (val << 4);
    }

    if(width > 8)  width = 8;

    return &rtnVal[8-width];
}

int adc_dig_chk(uint8 amux_ch, uint16 expected_val)
{
    uint16 result;
    int failure = 0;

    if (adc_meas(ANALOG_BASE, amux_ch, ANALOG_CH_TYPE_VOLTAGE, &result))
    {
#ifdef DEBUG_ANALOG
        uart_puts(UARTA_BASE, "\nadc_meas failed!\n");
#endif
        return(-1);
    }

    if (expected_val)
    {   if (result < ANALOG_DIG_HI_THRESH)
            failure = 1;
    }
    else
    {   if (result > ANALOG_DIG_LO_THRESH)
            failure = 1;
    }
    
    if (failure)
    {
#ifdef DEBUG_ANALOG
        uart_puts(UARTA_BASE, "\nmeas failed!\n");
        uart_puts(UARTA_BASE, "result=0x");
        uart_puts(UARTA_BASE, unsignedToAscii(result, 4));
        uart_puts(UARTA_BASE, "\nexp=");
        uart_puts(UARTA_BASE, unsignedToAscii(expected_val, 1));
        uart_puts(UARTA_BASE, "\namux_ch=0x");
        uart_puts(UARTA_BASE, unsignedToAscii(amux_ch, 2));
        uart_puts(UARTA_BASE, "\n");
#endif
        return(-1);
    }
    return(0);
}

uint16 div_shift(uint32 num, uint16 denom)
{
    uint16 result;

    switch (denom)
    {   case   1:   result = num;       break;
        case   2:   result = num >> 1;  break;
        case   4:   result = num >> 2;  break;
        case   8:   result = num >> 3;  break;
        case  16:   result = num >> 4;  break;
        case  32:   result = num >> 5;  break;
        case  64:   result = num >> 6;  break;
        case 128:   result = num >> 7;  break;
        default:    uart_puts(UARTA_BASE, "ERROR: avg not a power of 2!\n"); 
                    return(0);
    }
    return(result);
}

// this array gets initialized by meas_init:
struct internal_ch_desc mdata[MAX_CH*MAX_AVG+1];   // this is way oversized to avoid malloc

// the samples are stored here:
uint16 sample_storage_pool[MAX_CH*MAX_AVG];

int meas_init(struct analog_ch_desc *chan_desc)
{
    int i, j;
    uint16 inc;
    uint16 ndx;
    uint16 n_meas;
    uint16 n_chan;
    uint8 avg;
    uint16 *buf_ptr;
    uint16 *buf_start;

    n_chan = 0;
    n_meas = 0;

    // count number of channels & number of measurements
    for (i=0; i < MAX_CH; i++)
    {   if (chan_desc[i].amux_ch == AMUX_CH_NONE)
            break;
        else
        {   n_chan++;
            n_meas += chan_desc[i].avg_samples;
        }
    }

    if (i == MAX_CH)
    {
        uart_puts(UARTA_BASE, "mdata initialization failed! Too many channels!\n");
        return(0);
    }

    for (i=0; i < n_meas+1; i++)        // initialize & terminate the output array
        mdata[i].amux_ch = AMUX_CH_NONE;

    buf_ptr = &sample_storage_pool[0];

    // for all channels, scan input array
    for (i=0; i < n_chan; i++)
    {   // add measurement to output array

        buf_start = buf_ptr;

        // get required average samples for current measurement
        avg = chan_desc[i].avg_samples;

        // find increment into output array (= n_meas / avg)
        inc = div_shift((uint32) n_meas, (uint16) avg);

        // for all measurements in average group
        for (j=0; j < avg; j++)
        {   // disperse them evenly in output array

            // calculate optimal index
            ndx = j * inc;
            
            // while index position already used
            while ((mdata[ndx].amux_ch != AMUX_CH_NONE) && (ndx < n_meas))
                ndx++;  // increment to the next unused position

            if (ndx >= n_meas)
            {   uart_puts(UARTA_BASE, "mdata initialization failed! Ran off end of array!\n");
                return(0);
            }

            // stuff measurement descriptor entry
            mdata[ndx].amux_ch = chan_desc[i].amux_ch;
            mdata[ndx].ch_type = chan_desc[i].ch_type;
            mdata[ndx].meas_buf_start = buf_start;
            mdata[ndx].meas_buf_ptr   = buf_ptr++;
            mdata[ndx].result_ptr     = &chan_desc[i].result;
            if (j == (avg-1))   // if last one in averaging group
                mdata[ndx].avg_samples = avg;
            else
                mdata[ndx].avg_samples = 0;
            
        }
    }
        
    if (adc_init(ANALOG_BASE))
        uart_puts(UARTA_BASE, "ACM initialization failed!\n");

    return(n_meas);
}

void meas_display(uint16 meas)
{
    uart_puts(UARTA_BASE, unsignedToAscii(meas, 4));
    uart_puts(UARTA_BASE, "  ");
}

void meas_display_term(void)
{
    uart_puts(UARTA_BASE, "\r");
}

void bar_graph(uint16 result)
{
    // multiply result by 1.5 to take advantage of full range
    result = result + (result >> 1);

    // divide result by 8 to quantize to number of LEDs
    PUT_UINT32(GPIO0_BASE, (result >> 3) & 0x000000FF);
}


uint16 meas_avg(uint16 *meas_buf_ptr, uint8 avg_samples)
{
    int i;
    uint32 result;

    for (i=0, result = 0; i < avg_samples; i++)
        result += *meas_buf_ptr++;

    return(div_shift((uint32) result, (uint16) avg_samples));
}

void socMeasAnalogChans(struct analog_ch_desc *chan_desc)
{
    int i;
    int meas;
    int n_meas;
    uint8 n_samples;

    if (!(n_meas = meas_init(chan_desc)))
        return;

    uart_puts(UARTA_BASE, "\nM1AFS Analog Measurement Program v1.1\n");

    uart_puts(UARTA_BASE, "\nPOT   AT8   AT9   TINT  V1.5  I1.5  V2.5  I2.5  V3.3  I3.3\n");

    // continuously take & display measurements
    for (;;)
    {
        // take measurements
        for (meas = 0; meas < n_meas; meas++)
        {
            // if all measurements done
            if (mdata[meas].amux_ch == AMUX_CH_NONE)
                break;

            // take analog measurement
            adc_meas(ANALOG_BASE, mdata[meas].amux_ch, mdata[meas].ch_type, mdata[meas].meas_buf_ptr);

            n_samples = mdata[meas].avg_samples;

            // if no averaging or all averaging samples now available
            if (n_samples)
            {   // if averaging required
                if (n_samples != 1)
                    *mdata[meas].result_ptr = meas_avg(mdata[meas].meas_buf_start, n_samples);
                else
                    *mdata[meas].result_ptr = *mdata[meas].meas_buf_ptr;
            }
        }

        // display measurements
        for (i=0; i < MAX_CH; i++)
        {   if (chan_desc[i].amux_ch == AMUX_CH_NONE)
                break;
            else
                meas_display(chan_desc[i].result);  // display the result
        }

        // tell display all measurements done
        meas_display_term();

        // display first measurement on LEDs
        bar_graph(chan_desc[0].result);
    }
}

