/*******************************************************************************
 * (c) Copyright 2007 Actel Corporation.  All rights reserved.
 * 
 * Cortex-M1 vector table and boot code.
 * 
 * SVN $Revision: 1 $
 * SVN $Date: 2007-10-02 16:00:58 +0100 (Tue, 02 Oct 2007) $
 */

/*------------------------------------------------------------------------------
 * TOP_OF_MEMORY should be defined in your project's compiler settings in order
 * to specify the amount of RAM available on the target system. This is used to
 * locate the C stack at the top of memory.
 * A reasonable default value is provided in order to debug on available Actel
 * development boards.
 */
#ifndef TOP_OF_MEMORY
#define TOP_OF_MEMORY		0x00100000
#endif

/*------------------------------------------------------------------------------
 * main() function prototype as is called from this file.
 */
int main(void);

/*------------------------------------------------------------------------------
 * Exception handler / interrupt service routine function prototype typedef. 
 */
typedef void(* cortex_isr_addr_t)(void);

/*------------------------------------------------------------------------------
 * Exception handlers function prototypes.
 * The default implementation for these handlers can be found in
 * "default_handlers.c".
 */
void _start( void);
void reset_isr( void);
void nmi_isr( void );
void fault_isr( void );
void sv_call_isr( void );
void default_isr( void );
void pend_sv_isr( void );
void sys_tick_isr( void );

/*------------------------------------------------------------------------------
 * Symbols from the linker script used to locate the text, data and bss sections.
 */
extern unsigned int __text_load;
extern unsigned int __text_start;
extern unsigned int _etext;

extern unsigned int __data_load;
extern unsigned int __data_start;
extern unsigned int _edata;

extern unsigned int __bss_start__;
extern unsigned int __bss_end__;

/*------------------------------------------------------------------------------
 * Vector table.
 *	The vector table must be located at address 0x0.
 */
__attribute__ ((section(".isr_vector")))
cortex_isr_addr_t g_cortex_vectors[] =
{
	(cortex_isr_addr_t)TOP_OF_MEMORY,	/* Initial statck register value.*/
	_start,								/* Invoked on power up and warm reset.*/
	nmi_isr,							/* Non maskable interrupt.*/
	fault_isr,							/* All classes of fault. */
	0,0,0,0,0,0,0,						/* Reserved.*/
	sv_call_isr,						/* System service call with SVC instruction.*/
	0, 0,								/* Reserved.*/
	pend_sv_isr,						/* Pendable request for system service.*/
	sys_tick_isr,						/* System tick timer.*/ 
	default_isr,						/* External interrupt 0.*/
	default_isr,						/* External interrupt 1.*/
	default_isr,						/* External interrupt 2.*/
	default_isr,						/* External interrupt 3.*/
	default_isr,						/* External interrupt 4.*/
	default_isr,						/* External interrupt 5.*/
	default_isr,						/* External interrupt 6.*/
	default_isr							/* External interrupt 7.*/
};

/*------------------------------------------------------------------------------
 * _start() function called invoked on power up and warm reset.
 */
void _start( void)
{
	/*
	 * Copy text section if required (copy executable from LMA to VMA).
	 */
	{
		unsigned int * text_lma = &__text_load;
		unsigned int * end_text_lma = &_etext;
		unsigned int * text_vma = &__text_start;
		
		if ( text_vma != text_lma)
		{
			while ( text_lma <= end_text_lma)
			{
				*text_vma++ = *text_lma;
			}
		}
	}

	/*
	 * Copy data section if required (initialised variables).
	 */
	{
		unsigned int * data_lma = &__data_load;
		unsigned int * end_data_vma = &_edata;
		unsigned int * data_vma = &__data_start;
		
		if ( data_vma != data_lma )
		{
			while ( data_vma <= end_data_vma )
			{
				*data_vma++ = *data_lma++;
			}
		}
	}
	
	/*
	 * Zero out the bss section (set non-initialised variables to 0). 
	 */
	{
		unsigned int * bss = &__bss_start__;
		unsigned int * bss_end = &__bss_end__;
		
		if ( bss_end > bss)
		{
			while ( bss <= bss_end )
			{
				*bss++ = 0;
			}
		}
	}
	
	/*
	 * Call main.
	 */
	main();
	
	/*
	 * We should never return from _start().
	 */
	while( 1 )
	{
		;
	}
}
 
